#!/bin/sh
# the next line restarts using wish \
exec wish8.0 "$0" "$@"
#
#  - Roland A33 Midi sysex configuraion utility -
#
#   Required:   tcl/tk8.0x
#
#   Created:    07/00
#   Changed:    12/01 
#   Version:    1.2
#
#   Copyright (C) 2000-2001 Steffen Traeger
#	EMAIL:  Steffen.Traeger@t-online.de
#	URL:    http://home.t-online.de/home/Steffen.Traeger
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
########################################################################

#console show

# declare project directory
set execfile [file tail [info nameofexec]]
if { [string tolower $execfile] == "a33.exe" } {
	set projdir [file join [file dirname [info nameofexec]] ..]
} else {
	set projdir [file join [file dirname [info script]] ..]
}

#set tcl_library [file join [info nameofexec] lib tcl8.3]
lappend auto_path [file join $projdir lib midi]
package require midi 0.2

source patch.tcl
source map.tcl
source utils.tcl
source gui.tcl
source combo.tcl
catch {namespace import combobox::*}

set patchdir [file join $projdir patches]
set imagedir [file join $projdir images]

# default vars
set a33(Bank) "A"	;# A/B
set a33(Patch) 1	;# 1-16
set a33(oldBank) $a33(Bank)
set a33(oldPatch) $a33(Patch)
set a33(saved)	1
set a33(cancel)	0

set def(lever)    {Modulation Aftertouch}
set def(velocity) {Low Middle High}
set def(cc_name)  {Volume Velocity Modulation Expression Reverb Chorus}
set def(cc_value) {   7     128       1          11       91     93}
set def(transpose_name)  {-6 -5 -4 -3 -2 -1 0 1 2 3 4 5}
set def(transpose_value) {26 27 28 29 30 31 0  1  2  3  4  6}
set def(octave_name)  {-2 -1 0  1  2}
set def(octave_value) { 6  7 0  1  2}
set def(midiin) 0
set def(midiout) 0

# load further options
LoadDefFile

# change working directory
cd $patchdir

# default options
option add *Text.foreground			black
option add *Listbox.background		white
option add *Listbox.selectForeground	white  
option add *Entry.background			white
option add *Entry.selectBackground 		black
option add *Entry.selectForeground  	white
option add *borderWidth				2
option add *highlightThickness		0
option add *activeBorderWidth			0
option add *selectBorderWidth			0

# set palette under X
if { [string range [winfo server .] 0 0] == "X" } {
	set tk_strictMotif 1
	tk_setPalette LightGray
	set font(gen)				{Helvetica 12}
	set font(bold)				{Helvetica 12 bold}
	set font(panel)				{"Arial Narrow" 12}
	option add *Scrollbar.width		12
	option add *Listbox.selectBackground 	#000080
} else {
	set font(lcd)				{Helvetica 9 bold}
	set font(gen)				{Helvetica 8}
	set font(bold)				{Helvetica 8 bold}
	set font(panel)				{"Arial Narrow" 8}
	option add *Checkbutton.borderWidth	0
	option add *Radiobutton.borderWidth	0
}
option add *font $font(gen)

# main window
wm title . "A33"
wm iconname . A33
wm protocol . WM_DELETE_WINDOW "Quit"

# menu
menu .menu -tearoff 0 -bd 0 -relief flat
menu .menu.file -tearoff 0 
.menu add cascade -label "File" -menu .menu.file -underline 0
.menu.file add command -label "Save               " -underline 0 -command "SavePatch" -accelerator Ctrl+S
.menu.file add separator
.menu.file add command -label "Print To File      " -underline 0 -command "PrintToFile" -accelerator Ctrl+P
.menu.file add separator
.menu.file add command -label "Exit" -underline 0 -command Quit -accelerator Alt+X

menu .menu.edit -tearoff 0 
.menu add cascade -label "Edit" -menu .menu.edit -underline 0
.menu.edit add command -label "Cut Patch           " -underline 2 -command "CopyPatch; DeletePatch" -accelerator Ctrl+X
.menu.edit add command -label "Copy Patch" -underline 0 -command "CopyPatch" -accelerator Ctrl+C
.menu.edit add command -label "Paste Patch" -underline 0 -command "PastePatch" -accelerator Ctrl+V
.menu.edit add separator
.menu.edit add command -label "Delete Patch" -underline 0 -command "DeletePatch" -accelerator Ctrl+Y

menu .menu.midi -tearoff 0 
.menu add cascade -label "MIDI" -menu .menu.midi -underline 0
.menu.midi add command -label "Receive Patch       " -underline 0 -command "ReceiveMidi \$a33(Bank)-\$a33(Patch)"
.menu.midi add command -label "Receive All Patches  " -underline 0 -command "ReceiveMidi 0-0"
.menu.midi add separator
.menu.midi add command -label "Send Patch       " -underline 0 -command "SavePatch; SendMidi \$a33(Bank)-\$a33(Patch)"
.menu.midi add command -label "Send All Patches  " -underline 0 -command "SavePatch; SendMidi 0-0"

menu .menu.options -tearoff 0 
.menu add cascade -label "Options" -menu .menu.options -underline 0
.menu.options add cascade -label "MIDI IN  " -menu .menu.options.midiin -underline 0
.menu.options add cascade -label "MIDI OUT      " -menu .menu.options.midiout -underline 0 

# Get MIDI IN devices
menu .menu.options.midiin -tearoff 0
set i 0
foreach device [midi::getindevs] {
 	.menu.options.midiin add radio -label "  $device     " -variable def(midiin) -value $i -command SaveDefFile
	incr i
}

# Get MIDI OOU devices
menu .menu.options.midiout -tearoff 0
set i 0
foreach device [midi::getoutdevs] {
 	.menu.options.midiout add radio -label " $device     " -variable def(midiout) -value $i -command SaveDefFile
	incr i
}


menu .menu.help -tearoff 0 
.menu add cascade -label "Help" -menu .menu.help -underline 0
.menu.help add command -label "About         " -underline 0 -command About

. configure -menu .menu

# separator line
frame .sep1 -bd 1 -height 2 -relief sunken
  pack .sep1 -in . -side top -fill x

######################################################################################## 
#
# View area
#
frame .view -bd 1 -relief flat
  pack .view -in . -side top -expand 1 -fill both

# image definitions
image create photo but_off -file [file join $imagedir but_off.gif] -format gif
image create photo but_on -file [file join $imagedir but_on.gif] -format gif
image create photo check_off -file [file join $imagedir check_off.gif] -format gif
image create photo check_on -file [file join $imagedir check_on.gif] -format gif
image create photo but_rec -file [file join $imagedir but_rec.gif] -format gif
image create photo but_send -file [file join $imagedir but_send.gif] -format gif
image create photo but_cancel -file [file join $imagedir but_cancel.gif] -format gif
image create photo lower -file [file join $imagedir lower.gif] -format gif
image create photo upper -file [file join $imagedir upper.gif] -format gif
image create photo lon -file [file join $imagedir lon.gif] -format gif
image create photo loff -file [file join $imagedir loff.gif] -format gif
image create photo frontpanel -file [file join $imagedir frontpanel.gif] -format gif
image create photo lcd -file [file join $imagedir lcd.gif] -format gif
image create photo but_write -file [file join $imagedir but_write.gif] -format gif

frame .view.workspace -relief flat
  pack .view.workspace -in .view -side top -fill both

######################################################################################## 
#
# Select Bank/Patches
#
# canvas
set c .view.workspace.bp
canvas $c -relief flat -width 720 -height 140 -bg black -bd 1
  pack $c -in .view.workspace -side top
$c create image 0 0 -image frontpanel -anchor nw -tags item 

# Bank select A/B
CreateBankButtons $c 20 40

# Patch select 1-16
for {set i 1} {$i <= 8} {incr i} {
	CreatePatchButtons $c $i  20 40
}
for {set i 9} {$i <= 16} {incr i} {
	CreatePatchButtons $c $i 20 40
}
$c bind patchbutton <Any-Enter> "$c configure -cursor hand2"
$c bind patchbutton <Any-Leave> "$c configure -cursor {}"
$c bind patchbutton <1> "SelectPatch $c"

######################################################################################## 
#
# Common Parameters
#
# canvas
set c .view.workspace.com
canvas $c -relief flat -width 720 -height 140 -bg black -bd 1
  pack $c -in .view.workspace -side top
$c create image 0 0 -image frontpanel -anchor nw -tags item 

CreateCommonParams $c  20 10

######################################################################################## 
#
# Lower section
#
# canvas
set c .view.workspace.lower
canvas $c -relief flat -width 720 -height 140 -bg black -bd 1
$c create image 0 0 -image frontpanel -anchor nw -tags item 

CreateControls $c lower 20 10 

######################################################################################## 
#
# Upper section
#
# canvas
set c .view.workspace.upper
canvas $c -relief flat -width 720 -height 140 -bg black -bd 1
$c create image 0 0 -image frontpanel -anchor nw -tags item 

CreateControls $c upper 20 10 

##############################################################################
#
# quit program
#
proc Quit {} {
	destroy .
	exit 0
}

##############################################################################
#
# About box
#
proc About {} {
	set msg "  -- A33 1.1 --\n\
  Sysex Configuration Utility\n\
  for Roland A33 MIDI Controller\n\
  Copyright (C) 2000-2001, Steffen Trger\n\
      EMAIL:  Steffen.Traeger@t-online.de\n\
      URL:    http://home.t-online.de/home/Steffen.Traeger"

	tk_messageBox -parent . -title About -message $msg -type ok
}


############### main ###############################

.view.workspace.com.isLowerSelected invoke
.view.workspace.com.isUpperSelected invoke
update
LoadPatch

wm minsize . 1 1
wm maxsize . [winfo width .] [winfo height .]
