#
#  - GUI procedures -
#
#   Created:    07/00
#   Changed:    03/01 
#
########################################################################

##############################################################################
#
# Selects patch by mouse click
#
proc SelectPatch {c} {
	set patchname [lindex [$c gettags current] 0]
	set bank [lindex [split $patchname "-"] 0]		
	set patch [lindex [split $patchname "-"] 1]		

	$c.patch_A-$patch invoke
	$c.bank_$bank invoke
}

##############################################################################
#
# Update display messages
#
proc UpdateDisplay {{msg1 ""} {msg2 ""}} {
	.view.workspace.bp itemconfigure lcd1 -text [string toupper $msg1]
	.view.workspace.bp itemconfigure lcd2 -text [string toupper $msg2]
	update idletasks
	update
}

##############################################################################
#
# Count the "real" slide value
#
proc SetScaleValue {w var1 var2 op} {
global a33
	$w configure -text "[expr $a33($var2) * 8]"
}

##############################################################################
#
# Update Patch Names
#
proc SetPatchName {var1 var2 op} {
global a33

	.view.workspace.bp itemconfigure $a33(Bank)-$a33(Patch) -text $a33(name)
}

##############################################################################
#
# Toggles Upper section panel
#
proc ToggleUpper {} {
	
	if {[winfo viewable .view.workspace.upper]} {
		pack forget .view.workspace.upper
	} else {
		pack .view.workspace.upper -after .view.workspace.com -side top
	}
	update idletasks
}

##############################################################################
#
# Toggles Lower section panel
#
proc ToggleLower {} {

	if [winfo viewable .view.workspace.lower] {
		pack forget .view.workspace.lower
	} else {
		if {[winfo viewable .view.workspace.upper]} {
			pack .view.workspace.lower -after .view.workspace.upper
		} else {
			pack .view.workspace.lower -after .view.workspace.com -side top
		}
	}
	update idletasks
}

##############################################################################
#
# CreateBankButtons --
#
# Create a bank and data command buttons
#
# Arguments:    	c ... canvas window name
#			origin_x ... origins coordinates
#			origin_y ... origins coordinates
#
# Returns:    	-
#
proc CreateBankButtons {c origin_x origin_y} {
global a33 map font imagedir


	# offset
	set offset_x 0	
	set offset_y 0	

	# determine coordinates for bank buttons
	set x [expr $origin_x + $offset_x]	
	set y [expr $origin_y + $offset_y]

	# MIDI Write button
	$c create text [expr $x + 6] [expr $y - 8] -text "WRITE" -anchor w -fill #000000 -font $font(gen) -tag write
	scan [$c bbox write] "%s %s %s %s" x1 y1 x2 y2
	incr x1 -3
	incr y1 +1
	incr x2 +2
	incr y2 -1
	.view.workspace.bp create rectangle $x1 $y1 $x2 $y2 -fill #ffcc66 -width 0 -tag writebg
	.view.workspace.bp lower writebg write 
	button $c.send -bd 0 -relief flat -image but_write -bg #373b34 -activebackground #373b34 \
		-command "SavePatch; SendMidi \$a33(Bank)-\$a33(Patch)"
	  $c create window [expr $x + 0] [expr $y + 0] -window $c.send -anchor nw -tags item

	# Cancel button
	$c create text [expr $x + 0] [expr $y + 40] -text "STOP" -anchor w -fill #ffcc66 -font $font(gen)
	button $c.cancel -bd 0 -relief flat -image but_cancel -bg #373b34 -activebackground #373b34 \
		-command "set a33(cancel) 1"
	  $c create window [expr $x + 0] [expr $y + 48] -window $c.cancel -anchor nw -tags item

	# Saved check led
#	$c create text [expr $x + 50] [expr $y + 85] -text "MODIFIED" -anchor w -fill #ffcc66 -font $font(gen)
#	$c create image [expr $x + 34] [expr $y + 85] -image loff -anchor w -tags saved

	# Display
	$c create image [expr $x + 34] [expr $y + 42] -image lcd -anchor nw -tags item
	$c create text [expr $x + 40] [expr $y + 51] -text "" -anchor w -fill #cccc00 -font $font(lcd) -tags lcd1
	$c create text [expr $x + 40] [expr $y + 66] -text "" -anchor w -fill #cccc00 -font $font(lcd) -tags lcd2

	incr x 40 

	# Bank selection A/B
	radiobutton $c.bank_A -bd 0 -relief flat -image but_off -selectimage but_on -indicatoron 0 -bg #373b34 -activebackground #373b34 \
		-variable a33(Bank) -value "A" -selectcolor #373b34 -command "LoadPatch"
	  $c create window [expr $x + 20] $y -window $c.bank_A -anchor nw -tags item
	radiobutton $c.bank_B -bd 0 -relief flat -image but_off -selectimage but_on -indicatoron 0 -bg #373b34 -activebackground #373b34 \
		-variable a33(Bank) -value "B" -selectcolor #373b34 -command "LoadPatch"
	  $c create window [expr $x + 82] $y  -window $c.bank_B -anchor nw -tags item
	$c create text [expr $x + 72] [expr $y - 20] -text "PATCH" -anchor c -fill #c0c0c0 -font $font(gen)
	$c create line [expr $x + 20] [expr $y - 21] [expr $x + 50] [expr $y - 21] -fill #c0c0c0 
	$c create line [expr $x + 93] [expr $y - 21] [expr $x + 122] [expr $y - 21] -fill #c0c0c0 
	$c create text [expr $x + 40] [expr $y - 8] -text "A" -anchor c -fill #ffcc66 -font $font(gen)
	$c create text [expr $x + 100] [expr $y - 8] -text "B" -anchor c -fill #ffcc66 -font $font(gen)

}

##############################################################################
#
# CreatePatchButtons --
#
# Create a patch, bank and data command buttons
#
# Arguments:    	c ... canvas window name
#			nr ... patch number
#			origin_x ... origins coordinates
#			origin_y ... origins coordinates
#
# Returns:    	-
#
proc CreatePatchButtons {c nr origin_x origin_y} {
global a33 map font imagedir

	# offset patch related symbols
	set offset_x 200
	set offset_y 0

	# determine coordinates for patch buttons
	set x [expr $origin_x + $offset_x + 62 * ( ($nr - 1) % 8 )]
	set y [expr $origin_y + $offset_y + ($nr - 1) / 8 * 60 ]

	# create buttons and marks
	if { $nr == "1" } {

		# symbols and lines
		image create photo patchnr -file [file join $imagedir patchnr.gif] -format gif
		image create photo arrow -file [file join $imagedir arrow.gif] -format gif
#		$c create image $x [expr $y + 17] -image patchnr -anchor nw -tags item 
#		$c create image $x [expr $y + 77] -image patchnr -anchor nw -tags item 

		$c create text [expr $x - 8] [expr $y - 19] -text "A" -anchor e -fill #c0c0c0  -font $font(gen)
		$c create image [expr $x - 8] [expr $y - 19] -image arrow -anchor w -tags item 

		$c create text [expr $x - 8] [expr $y - 7] -text "B" -anchor e -fill #c0c0c0  -font $font(gen)
		$c create image [expr $x - 8] [expr $y - 7] -image arrow -anchor w -tags item 

		$c create text [expr $x - 8] [expr $y + 41] -text "A" -anchor e -fill #c0c0c0  -font $font(gen)
		$c create image [expr $x - 8] [expr $y + 41] -image arrow -anchor w -tags item 

		$c create text [expr $x - 8] [expr $y + 53] -text "B" -anchor e -fill #c0c0c0  -font $font(gen)
		$c create image [expr $x - 8] [expr $y + 53] -image arrow -anchor w -tags item 

	}

	# now the buttons
	$c create text [expr $x + 22] [expr $y + 23] -text $nr -anchor c -fill #c0c0c0 -font $font(gen)

	LoadMap A-$nr
	$c create text [expr $x + 2] [expr $y - 20] -text $map(A-$nr,name) -anchor w -fill #ffcc66 -font $font(panel) -tags "A-$nr patchbutton"
	
	LoadMap B-$nr
	$c create text [expr $x + 2] [expr $y - 8] -text $map(B-$nr,name) -anchor w -fill #ffcc66 -font $font(panel) -tags "B-$nr patchbutton"

	radiobutton $c.patch_A-$nr -bd 0 -relief flat -image but_off -selectimage but_on -indicatoron 0 -bg #373b34 -activebackground #373b34 \
 		-variable a33(Patch) -value $nr -selectcolor #373b34 -command "LoadPatch"
	$c create window $x $y -window $c.patch_A-$nr -anchor nw -tags item


}

##############################################################################
#
# CreateCommonParams --
#
# Create common parameter widgets
#
# Arguments:    	c ... canvas window name
#			origin_x ... origins coordinates
#			origin_y ... origins coordinates
#
# Returns:    	-
#
proc CreateCommonParams {c origin_x origin_y} {
global a33 map font imagedir def

	# offset patch related symbols
	set offset_x 0	
	set offset_y 10

	# determine coordinates for patch buttons
	set x [expr $origin_x + $offset_x ]	
	set y [expr $origin_y + $offset_y]

	# MIDI Out
	$c create text $x [expr $y + 0] -text "MIDI" -anchor w -fill #c0c0c0  -font $font(gen)
	$c create text $x [expr $y + 20] -text "OUT1" -anchor w -fill #ffcc66  -font $font(gen)
	checkbutton $c.isMidiOut1 -bd 0 -relief flat -image check_off -selectimage check_on -indicatoron 0 -bg #373e34 -activebackground #373e34 \
		-variable a33(isMidiOut1) -selectcolor #373e34 
	$c create window $x [expr $y + 29] -window $c.isMidiOut1 -anchor nw -tags item
	$c create text $x [expr $y + 60] -text "OUT2" -anchor w -fill #ffcc66  -font $font(gen)
	checkbutton $c.isMidiOut2 -bd 0 -relief flat -image check_off -selectimage check_on -indicatoron 0 -bg #373e34 -activebackground #373e34 \
		-variable a33(isMidiOut2) -selectcolor #373e34 
	$c create window $x [expr $y + 69] -window $c.isMidiOut2 -anchor nw -tags item

	# Separator
	$c create line [expr $x + 35] [expr $y - 5] [expr $x + 35] [expr $y + 110] -fill #c0c0c0 -dash .
	
	set x [expr $x + 50]

	# Name
	$c create text $x [expr $y + 0] -text "NAME" -anchor w -fill #ffcc66 -font $font(gen)
	entry $c.name -textvariable a33(name) -width 15 -insertbackground white -bg black -fg white
	  $c create window $x [expr $y + 8] -window $c.name -anchor nw -tags item
	trace variable a33(name) w "SetPatchName"
	# Lever
	$c create text $x [expr $y + 40] -text "LEVER TO" -anchor w -fill #ffcc66 -font $font(gen)
	combobox $c.lever -textvariable a33(lever) -editable false -bg black -fg white -selectbackground #c0c0c0 -selectforeground black \
			-highlightthickness 0 -width 12 
   	eval $c.lever list insert end $def(lever)
	  $c create window $x [expr $y + 48] -window $c.lever -anchor nw -tags item
	# Velocity
	$c create text $x [expr $y + 80] -text "VELOCITY" -anchor w -fill #ffcc66 -font $font(gen)
	combobox $c.velocity -textvariable a33(velocity) -editable false -bg black -fg white -selectbackground #c0c0c0 -selectforeground black \
			-highlightthickness 0 -width 12 
   	eval $c.velocity list insert end $def(velocity)
	  $c create window $x [expr $y + 88] -window $c.velocity -anchor nw -tags item

	set x [expr $x + 120]

	# Description
	$c create text $x [expr $y + 0] -text "DESCRIPTION" -anchor w -fill #ffcc66 -font $font(gen)
	entry $c.description -textvariable a33(description) -width 40 -insertbackground white -bg black -fg white
	  $c create window $x [expr $y + 8] -window $c.description -anchor nw -tags item
	# Data Entry
	$c create text $x [expr $y + 40] -text "DATA ENTRY TO" -anchor w -fill #ffcc66 -font $font(gen)
	combobox $c.data_entry -textvariable a33(_data_entry) -editable false -bg black -fg white -selectbackground #c0c0c0 \
		-selectforeground black -highlightthickness 0 -width 12 
	eval $c.data_entry list insert end $def(cc_name)
	  $c create window $x [expr $y + 48] -window $c.data_entry -anchor nw -tags item
	# Transpose
	$c create text $x [expr $y + 80] -text "TRANSPOSE VALUE" -anchor w -fill #ffcc66 -font $font(gen)
	scale $c.transpose -orient horizontal -length 100 -from -6 -to 5 -showvalue 0 -bg #373e34 -highlightbackground black \
		-activebackground #c0c0c0 -bd 2 -troughcolor black \
		 -tickinterval 0 -variable a33(_transpose) -width 12 -sliderlength 20
	  $c create window $x [expr $y + 88] -window $c.transpose -anchor nw -tags item
	entry $c.e_transpose -bd 1 -relief flat -textvariable a33(transpose) -width 3 -insertbackground white -bg #373e34 \
		-fg white -state disabled -justify c
	  $c create window [expr $x + 101] [expr $y + 88] -window $c.e_transpose -anchor nw -tags item
	# isTransposed
	$c create text [expr $x + 150] [expr $y + 80] -text "TRANSPOSE" -anchor w -fill #ffcc66  -font $font(gen)
	checkbutton $c.isTransposed -bd 0 -relief flat -image check_off -selectimage check_on -indicatoron 0 -bg #373e34 \
		-activebackground #373e34 -variable a33(isTransposed) -selectcolor #373e34 
	$c create window [expr $x + 170] [expr $y + 90] -window $c.isTransposed -anchor nw -tags item

	# Separator
	$c create line [expr $x + 260] [expr $y - 5] [expr $x + 260] [expr $y + 110] -fill #c0c0c0 -dash .
	
	set x [expr $x + 280]

	# Keyboard Mode
	$c create text [expr $x + 28] [expr $y + 0] -text "KEYBOARD MODE" -anchor w -fill #c0c0c0  -font $font(gen)
	$c create line [expr $x + 0] [expr $y + 0] [expr $x + 24] [expr $y + 0] -fill #c0c0c0 
	$c create line [expr $x + 120] [expr $y + 0] [expr $x + 145] [expr $y + 0] -fill #c0c0c0 
	$c create text $x [expr $y + 20] -text "LOWER" -anchor w -fill #ffcc66  -font $font(gen)
	checkbutton $c.isLower -bd 0 -relief flat -image check_off -selectimage check_on -indicatoron 0 -bg #373e34 -activebackground #373e34 \
		-variable a33(isLower) -selectcolor #373e34 -command "set a33(isSplitted) 0"
	$c create window $x [expr $y + 29] -window $c.isLower -anchor nw -tags item
	$c create text [expr $x + 60] [expr $y + 20] -text "UPPER" -anchor w -fill #ffcc66  -font $font(gen)
	checkbutton $c.isUpper -bd 0 -relief flat -image check_off -selectimage check_on -indicatoron 0 -bg #373e34 -activebackground #373e34 \
		-variable a33(isUpper) -selectcolor #373e34 -command "set a33(isSplitted) 0"
	$c create window [expr $x + 60] [expr $y + 29] -window $c.isUpper -anchor nw -tags item
	$c create text [expr $x + 120] [expr $y + 20] -text "SPLIT" -anchor w -fill #ffcc66  -font $font(gen)
	checkbutton $c.isSplitted -bd 0 -relief flat -image check_off -selectimage check_on -indicatoron 0 -bg #373e34 -activebackground #373e34 \
		-variable a33(isSplitted) -selectcolor #373e34 -command "set a33(isLower) 1; set a33(isUpper) 1"
	$c create window [expr $x + 120] [expr $y + 29] -window $c.isSplitted -anchor nw -tags item
	
	# SplitKey
	$c create text [expr $x + 170] [expr $y + 20] -text "SPLIT KEY" -anchor w -fill #ffcc66  -font $font(gen)
	entry $c.split -bd 2 -relief raised -textvariable a33(split) -width 4 -insertbackground white -bg #373e34 \
		-fg white -state disabled -justify c
	  $c create window [expr $x + 170] [expr $y + 29] -window $c.split -anchor nw -tags item
	$c create line [expr $x + 190] [expr $y + 38] [expr $x + 215] [expr $y + 38] -fill #ffcc66 
	$c create line [expr $x + 215] [expr $y + 38] [expr $x + 215] [expr $y + 95] -fill #ffcc66 
	$c create line [expr $x + 190] [expr $y + 95] [expr $x + 215] [expr $y + 95] -fill #ffcc66 
	
	set y [expr $y + 60]

	# Select
	$c create text [expr $x + 28] [expr $y + 0] -text "VIEW" -anchor w -fill #c0c0c0  -font $font(gen)
	$c create line [expr $x + 0] [expr $y + 0] [expr $x + 23] [expr $y + 0] -fill #c0c0c0 
	$c create line [expr $x + 61] [expr $y + 0] [expr $x + 90] [expr $y + 0] -fill #c0c0c0 
	$c create text $x [expr $y + 20] -text "LOWER" -anchor w -fill #ffcc66  -font $font(gen)
	checkbutton $c.isLowerSelected -bd 0 -relief flat -image check_off -selectimage check_on -indicatoron 0 -bg #373e34 \
		 -activebackground #373e34 -variable a33(isLowerSelected) -selectcolor #373e34 -command "ToggleLower"
	$c create window $x [expr $y + 29] -window $c.isLowerSelected -anchor nw -tags item
	$c create text [expr $x + 60] [expr $y + 20] -text "UPPER" -anchor w -fill #ffcc66  -font $font(gen)
	checkbutton $c.isUpperSelected -bd 0 -relief flat -image check_off -selectimage check_on -indicatoron 0 -bg #373e34 \
		-activebackground #373e34 -variable a33(isUpperSelected) -selectcolor #373e34 -command "ToggleUpper"
	$c create window [expr $x + 60] [expr $y + 29] -window $c.isUpperSelected -anchor nw -tags item

	# Select SplitKey
	$c create text [expr $x + 170] [expr $y + 20] -text "SELECT" -anchor w -fill #ffcc66  -font $font(gen)
	button $c.keyboard -bd 0 -relief flat -image check_off -bg #373b34 -activebackground #373b34 \
		-command "SelectKeyDialog"
	$c create window [expr $x + 170] [expr $y + 29] -window $c.keyboard -anchor nw -tags item
}

##############################################################################
#
# CreateControls --
#
# Create control parameter widgets
#
# Arguments:    	c ... canvas window name
#			lower|upper ... keyboard section
#			origin_x ... origins coordinates
#			origin_y ... origins coordinates
#
# Returns:    	-
#
proc CreateControls {c section origin_x origin_y} {
global a33 map font imagedir def

	# offset patch related symbols
	set offset_x 0	
	set offset_y 10

	# determine coordinates for patch buttons
	set x [expr $origin_x + $offset_x ]	
	set y [expr $origin_y + $offset_y]

	# Patch Name
	$c create text $x [expr $y + 0] -text "[string toupper $section] PATCH" -anchor w -fill #ffcc66 -font $font(gen)
	entry $c.name -textvariable a33(${section}_name) -width 27 -insertbackground white -bg black -fg white
	  $c create window $x [expr $y + 8] -window $c.name -anchor nw -tags item
	# Channel
	$c create text $x [expr $y + 40] -text "CHANNEL (1-16)" -anchor w -fill #ffcc66 -font $font(gen)
	entry $c.channel -textvariable a33(${section}_midichannel) -width 12 -insertbackground white -bg black -fg white
	  $c create window $x [expr $y + 48] -window $c.channel -anchor nw -tags item
	# Programm Change
	$c create text $x [expr $y + 80] -text "PROG. (1-128)" -anchor w -fill #ffcc66 -font $font(gen)
	entry $c.program -textvariable a33(${section}_program) -width 12 -insertbackground white -bg black -fg white
	  $c create window $x [expr $y + 88] -window $c.program -anchor nw -tags item

	set x [expr $x + 90]

	# CC-00
	$c create text $x [expr $y + 40] -text "CC-00 (0-127)" -anchor w -fill #ffcc66 -font $font(gen)
	entry $c.cc00 -textvariable a33(${section}_cc00) -width 12 -insertbackground white -bg black -fg white
	  $c create window $x [expr $y + 48] -window $c.cc00 -anchor nw -tags item
	# CC_32
	$c create text $x [expr $y + 80] -text "CC-32 (0-127)" -anchor w -fill #ffcc66 -font $font(gen)
	entry $c.cc32 -textvariable a33(${section}_cc32) -width 12 -insertbackground white -bg black -fg white
	  $c create window $x [expr $y + 88] -window $c.cc32 -anchor nw -tags item

	# Separator
	$c create line [expr $x + 105] [expr $y - 5] [expr $x + 105] [expr $y + 110] -fill #c0c0c0 -dash .

	set x [expr $x + 140]

	# measure
	$c create line [expr $x - 8] [expr $y + 12] [expr $x + 33] [expr $y + 12] -fill #c0c0c0
	$c create line [expr $x - 8] [expr $y + 21] [expr $x + 33] [expr $y + 21] -fill #c0c0c0
	$c create line [expr $x - 8] [expr $y + 30] [expr $x + 33] [expr $y + 30] -fill #c0c0c0
	$c create line [expr $x - 10] [expr $y + 39] [expr $x + 35] [expr $y + 39] -fill #c0c0c0 -width 2
	$c create line [expr $x - 8] [expr $y + 48] [expr $x + 33] [expr $y + 48] -fill #c0c0c0
	$c create line [expr $x - 8] [expr $y + 57] [expr $x + 33] [expr $y + 57] -fill #c0c0c0
	$c create line [expr $x - 8] [expr $y + 65] [expr $x + 33] [expr $y + 65] -fill #c0c0c0
	$c create line [expr $x - 8] [expr $y + 74] [expr $x + 33] [expr $y + 74] -fill #c0c0c0
	$c create line [expr $x - 8] [expr $y + 83] [expr $x + 33] [expr $y + 83] -fill #c0c0c0

	# Volume
	$c create text $x [expr $y + 0] -text "VOL" -anchor w -fill #ffcc66 -font $font(gen)
	scale $c.volume -orient vertical -length 82 -from 15 -to 0 -showvalue 0 -bg #373e34 -highlightbackground black \
		-activebackground #c0c0c0 -bd 2 -troughcolor black \
		 -tickinterval 0 -variable a33(${section}_volume) -width 16 -sliderlength 12
	  $c create window $x [expr $y + 8] -window $c.volume -anchor nw -tags item
	label $c.l_volume -bd 0 -relief flat -width 3 -bg #373e34 -fg white -justify c
	  $c create window [expr $x + 2] [expr $y + 94] -window $c.l_volume -anchor nw -tags item
	trace variable a33(${section}_volume) w "SetScaleValue $c.l_volume"

	set x [expr $x + 50]

	# measure
	$c create line [expr $x - 8] [expr $y + 12] [expr $x + 33] [expr $y + 12] -fill #c0c0c0
	$c create line [expr $x - 8] [expr $y + 21] [expr $x + 33] [expr $y + 21] -fill #c0c0c0
	$c create line [expr $x - 8] [expr $y + 30] [expr $x + 33] [expr $y + 30] -fill #c0c0c0
	$c create line [expr $x - 10] [expr $y + 39] [expr $x + 35] [expr $y + 39] -fill #c0c0c0 -width 2
	$c create line [expr $x - 8] [expr $y + 48] [expr $x + 33] [expr $y + 48] -fill #c0c0c0
	$c create line [expr $x - 8] [expr $y + 57] [expr $x + 33] [expr $y + 57] -fill #c0c0c0
	$c create line [expr $x - 8] [expr $y + 65] [expr $x + 33] [expr $y + 65] -fill #c0c0c0
	$c create line [expr $x - 8] [expr $y + 74] [expr $x + 33] [expr $y + 74] -fill #c0c0c0
	$c create line [expr $x - 8] [expr $y + 83] [expr $x + 33] [expr $y + 83] -fill #c0c0c0

	# Chorus
	$c create text $x [expr $y + 0] -text "CHO" -anchor w -fill #ffcc66 -font $font(gen)
	scale $c.chorus -orient vertical -length 82 -from 15 -to 0 -showvalue 0 -bg #373e34 -highlightbackground black \
		-activebackground #c0c0c0 -bd 2 -troughcolor black \
		 -tickinterval 0 -variable a33(${section}_chorus) -width 16 -sliderlength 12
	  $c create window $x [expr $y + 8] -window $c.chorus -anchor nw -tags item
	label $c.l_chorus -bd 0 -relief flat -width 3 -bg #373e34 -fg white -justify c
	  $c create window [expr $x + 2] [expr $y + 94] -window $c.l_chorus -anchor nw -tags item
	trace variable a33(${section}_chorus) w "SetScaleValue $c.l_chorus"

	set x [expr $x + 50]

	# measure
	$c create line [expr $x - 8] [expr $y + 12] [expr $x + 33] [expr $y + 12] -fill #c0c0c0
	$c create line [expr $x - 8] [expr $y + 21] [expr $x + 33] [expr $y + 21] -fill #c0c0c0
	$c create line [expr $x - 8] [expr $y + 30] [expr $x + 33] [expr $y + 30] -fill #c0c0c0
	$c create line [expr $x - 10] [expr $y + 39] [expr $x + 35] [expr $y + 39] -fill #c0c0c0 -width 2
	$c create line [expr $x - 8] [expr $y + 48] [expr $x + 33] [expr $y + 48] -fill #c0c0c0
	$c create line [expr $x - 8] [expr $y + 57] [expr $x + 33] [expr $y + 57] -fill #c0c0c0
	$c create line [expr $x - 8] [expr $y + 65] [expr $x + 33] [expr $y + 65] -fill #c0c0c0
	$c create line [expr $x - 8] [expr $y + 74] [expr $x + 33] [expr $y + 74] -fill #c0c0c0
	$c create line [expr $x - 8] [expr $y + 83] [expr $x + 33] [expr $y + 83] -fill #c0c0c0

	# Reverb
	$c create text $x [expr $y + 0] -text "REV" -anchor w -fill #ffcc66 -font $font(gen)
	scale $c.reverb -orient vertical -length 82 -from 15 -to 0 -showvalue 0 -bg #373e34 -highlightbackground black \
		-activebackground #c0c0c0 -bd 2 -troughcolor black \
		 -tickinterval 0 -variable a33(${section}_reverb) -width 16 -sliderlength 12
	  $c create window $x [expr $y + 8] -window $c.reverb -anchor nw -tags item
	label $c.l_reverb -bd 0 -relief flat -width 3 -bg #373e34 -fg white -justify c
	  $c create window [expr $x + 2] [expr $y + 94] -window $c.l_reverb -anchor nw -tags item
	trace variable a33(${section}_reverb) w "SetScaleValue $c.l_reverb"

	# Octave
	set x [expr $x + 50]
	$c create text $x [expr $y + 0] -text "OCT" -anchor w -fill #ffcc66 -font $font(gen)
	scale $c.octave -orient vertical -length 62 -from 2 -to -2 -showvalue 0 -bg #373e34 -highlightbackground black \
		-activebackground #c0c0c0 -bd 2 -troughcolor black \
		 -tickinterval 0 -variable a33(${section}_oct) -width 16 -sliderlength 12
	  $c create window $x [expr $y + 8] -window $c.octave -anchor nw -tags item
	entry $c.e_octave -bd 0 -relief flat -textvariable a33(${section}_oct) -width 3 -insertbackground white -bg #373e34  \
		-fg white -state disabled -justify c
	  $c create window [expr $x + 2] [expr $y + 74] -window $c.e_octave -anchor nw -tags item
	$c create line [expr $x - 8] [expr $y + 17] [expr $x + 33] [expr $y + 17] -fill #c0c0c0
	$c create line [expr $x - 8] [expr $y + 29] [expr $x + 33] [expr $y + 29] -fill #c0c0c0
	$c create line [expr $x - 10] [expr $y + 41] [expr $x + 35] [expr $y + 41] -fill #c0c0c0 -width 2
	$c create line [expr $x - 8] [expr $y + 53] [expr $x + 33] [expr $y + 53] -fill #c0c0c0
	$c create line [expr $x - 8] [expr $y + 65] [expr $x + 33] [expr $y + 65] -fill #c0c0c0

	# Separator
	$c create line [expr $x + 50] [expr $y - 5] [expr $x + 50] [expr $y + 110] -fill #c0c0c0 -dash .

	#
	# Effects
	# 

	# Chorus
	set x [expr $x + 70]
	$c create text [expr $x + 62] [expr $y + 0] -text "EFFECTS" -anchor w -fill #c0c0c0  -font $font(gen)
	$c create line [expr $x + 0] [expr $y + 0] [expr $x + 56] [expr $y + 0] -fill #c0c0c0 
	$c create line [expr $x + 110] [expr $y + 0] [expr $x + 175] [expr $y + 0] -fill #c0c0c0 

	$c create text $x [expr $y + 20] -text "CHORUS" -anchor w -fill #ffcc66  -font $font(gen)
	checkbutton $c.isChorus -bd 0 -relief flat -image check_off -selectimage check_on -indicatoron 0 -bg #373e34 -activebackground #373e34 \
		-variable a33(${section}_isChorus) -selectcolor #373e34 
	$c create window $x [expr $y + 29] -window $c.isChorus -anchor nw -tags item
	# Reverb
	$c create text $x [expr $y + 60] -text "REVERB" -anchor w -fill #ffcc66  -font $font(gen)
	checkbutton $c.isReverb -bd 0 -relief flat -image check_off -selectimage check_on -indicatoron 0 -bg #373e34 -activebackground #373e34 \
		-variable a33(${section}_isReverb) -selectcolor #373e34 
	$c create window $x [expr $y + 69] -window $c.isReverb -anchor nw -tags item

	# Sustain
	set x [expr $x + 60]
	$c create text $x [expr $y + 20] -text "SUSTAIN" -anchor w -fill #ffcc66  -font $font(gen)
	checkbutton $c.isSustain -bd 0 -relief flat -image check_off -selectimage check_on -indicatoron 0 -bg #373e34 -activebackground #373e34 \
		-variable a33(${section}_isSustain) -selectcolor #373e34 
	$c create window $x [expr $y + 29] -window $c.isSustain -anchor nw -tags item
	# Expression
	$c create text $x [expr $y + 60] -text "EXPRESSION" -anchor w -fill #ffcc66  -font $font(gen)
	checkbutton $c.isExpression -bd 0 -relief flat -image check_off -selectimage check_on -indicatoron 0 -bg #373e34 \
		-activebackground #373e34 -variable a33(${section}_isExpression) -selectcolor #373e34 
	$c create window $x [expr $y + 69] -window $c.isExpression -anchor nw -tags item

	# Pitch Bend
	set x [expr $x + 60]
	$c create text $x [expr $y + 20] -text "PITCH BEND" -anchor w -fill #ffcc66  -font $font(gen)
	checkbutton $c.isPBLever -bd 0 -relief flat -image check_off -selectimage check_on -indicatoron 0 -bg #373e34 -activebackground #373e34 \
		-variable a33(${section}_isPBLever) -selectcolor #373e34 
	$c create window $x [expr $y + 29] -window $c.isPBLever -anchor nw -tags item

	# sign
	$c create image [expr $x + 90] [expr $y - 10] -image $section -anchor nw -tags item

}

#############################################################################
#
# SelectKey --
#
# Select split key
#
# Arguments:    c ... canvas
#
# Returns:    -
#
proc SelectKey {c {key {}}} {
global a33

	# unallowed keys
	if { $key == {} } {
		set key [lindex [$c gettags current] 0]
	}	
	regexp "\[0-9\].*" $key num	
	if { ($num < 11) || ($num > 72) } {
		return
	}
	set a33(tmp_split) [expr $num - 11]

	# delete old selection
	set old_key [$c find withtag selected]
	if { $old_key != {} } {
		set color [lindex [$c gettags selected] 1]
		$c itemconfigure $old_key -fill $color
		$c dtag $old_key selected
	}

	# select new
	$c itemconfigure $key -fill #669999
	$c addtag selected withtag $key
}

##############################################################################
#
# DrawKeyboard --
#
# Draw a keyboard to determine the split point
#
# Arguments:    c ... canvas
#
# Returns:    -
#
proc DrawKeyboard {c} {

	set kw 16
	set kh 80
	$c configure -width [expr 45 * $kw + 1]
	$c configure -height [expr $kh + 1]

    	set whiteIDs { 0 2 4 5 7 9 11 }

	set x -47
	for {set i 0} {$i < 7} {incr i} {
		for {set j 0} {$j < 7} {incr j} {
			set x [expr $x + $kw]
			set keyNum [expr $i * 12 + [lindex $whiteIDs $j]]
			AddKey white  $c $x 1 $kw $kh $keyNum
		}
	}

	set x -47
	for {set i 0} {$i < 7} {incr i} {
		set x [expr $x + $kw]
		set keyNum [expr $i * 12]
		AddKey black $c [set x [expr $x + $kw - 3]] 1 [expr $kw/2] [expr $kh/2] [expr $keyNum + 1]
		AddKey black $c [set x [expr $x + $kw]] 1 [expr $kw/2] [expr $kh/2] [expr $keyNum + 3]
		set x [expr $x + $kw]
		AddKey black $c [set x [expr $x + $kw]] 1 [expr $kw/2] [expr $kh/2] [expr $keyNum + 6]
		AddKey black $c [set x [expr $x + $kw]] 1 [expr $kw/2] [expr $kh/2] [expr $keyNum + 8]
		AddKey black $c [set x [expr $x + $kw]] 1 [expr $kw/2] [expr $kh/2] [expr $keyNum + 10]
		incr x 3
	}
	$c bind all <1> "SelectKey $c"
}

##############################################################################
#
# AddWhiteKey --
#
# Draw a key tagged with the key number
#
# Arguments:    color ... black|white
#				c ... canvas
#				x, y, width, height ... rectangle coordinates
#				num ... key number
#
# Returns:    -
#
proc AddKey {color c x y width height num} {

	# A33 view
	if { ($num < 4) || ($num > 79) } {
		return
	}

	if { ($num < 11) || ($num > 72) } {
		set fillcolor #cfcfcf
	} else {
		set fillcolor $color
	}

	$c create rectangle $x $y [expr $x + $width] [expr $y + $height] -outline black -fill $fillcolor -width 1 -tag "k$num $color"
}

##############################################################################
#
# SelectKeyDialog --
#
# Dialog to select the split key 
#
# Arguments:    color ... black|white
#				c ... canvas
#				x, y, width, height ... rectangle coordinates
#				num ... key number
#
# Returns:    -
#
proc SelectKeyDialog {} {
global a33
	# new window
	set w .keyboard

	catch {destroy $w}
 	toplevel $w -bd 0
	wm title $w "Select split key"
    wm iconname $w "Split"
	wm minsize $w 10 10

	frame $w.control -bd 1 -relief flat 
	  pack $w.control -in $w -side bottom -fill x
	frame $w.frame -bd 1 -relief flat 
	  pack $w.frame -in $w -side top -expand 1 -fill both -padx 5 -pady 5
	frame $w.sep -bd 1 -height 2 -relief groove
  	  pack $w.sep -in $w -side top -fill x

	label $w.frame.label -text "     Please select the split key:" -width 12 -anchor w
	  pack $w.frame.label -in $w.frame -side top -fill x
	canvas $w.frame.c -relief flat -borderwidth 1
	  pack $w.frame.c -in $w.frame -side top -fill x -padx 10
	DrawKeyboard $w.frame.c 

	button $w.control.ok -width 12 -text "OK" -under 0 -command "set a33(split) \$a33(tmp_split); destroy $w"
	  pack $w.control.ok -in $w.control -side left -expand 1 -padx 3m -pady 2m
	button $w.control.quit -width 12 -text "Cancel" -under 0 -command "destroy $w"
	  pack $w.control.quit -in $w.control -side left -expand 1 -padx 3m -pady 2m

	wm withdraw $w
	update idletasks
	set x [expr [winfo x .] + ([winfo width .] / 2) - ([winfo reqwidth $w] / 2)]
	set y [expr [winfo y .] + ([winfo height .] / 2) - ([winfo reqheight $w] / 2)]
	wm geometry $w +$x+$y
	update idletasks
	wm deiconify $w
	update idletasks

	SelectKey $w.frame.c "k[expr $a33(split) + 11]"
}
